//
//  map.h
//
//  Created by Mashpoe on 1/15/21.
//  Modifications by BiglyDerv
//

#ifndef map_h
#define map_h

#define hashmap_str_lit(str) (str), sizeof(str) - 1
#define hashmap_static_arr(arr) (arr), sizeof(arr)

#include <stddef.h>
#include <stdint.h>

struct bucket {
  // `next` must be the first struct element.
  // changing the order will break multiple functions
  struct bucket *next;

  // key, key size, key hash, and associated value
  const void *key;
  size_t ksize;
  uint32_t hash;
  uintptr_t value;
};

struct hashmap {
  struct bucket *buckets;
  int capacity;
  int count;

  // "tombstones" are empty buckets from removing elements
  int tombstone_count;

  // a linked list of all valid entries, in order
  struct bucket *first;
  // lets us know where to add the next element
  struct bucket *last;
};

typedef struct hashmap hashmap;

typedef int (*hashmap_callback)(const void *key, size_t ksize, uintptr_t value,
                                void *usr);

hashmap *hashmap_create(void);
void hashmap_free(hashmap *map);
int hashmap_set(hashmap *map, const void *key, size_t ksize, uintptr_t value,
                int free_old);
int hashmap_get(hashmap *map, const void *key, size_t ksize,
                uintptr_t *out_val);
int hashmap_size(hashmap *map);
int hashmap_iterate(hashmap *map, hashmap_callback c, void *usr);
#endif // map_h
